@@ -173,7 +173,7 @@ void ComicApplet::dataUpdated(const ComicMetaData &data)
         return;
     }
 
-    setConfigurationRequired(false);
+    setConfigurationRequired(mCurrent.id().isEmpty());
 
     // there was an error, display information as image
     if (data.error) {
@@ -485,9 +485,9 @@ void ComicApplet::updateComic(const QString &identifierSuffix)
         mEngine->requestSource(identifier);
         slotScaleToContent();
     } else {
+        setBusy(false);
         qWarning() << "Either no identifier was specified or the engine could not be created:"
                    << "id" << id;
-        setConfigurationRequired(true);
     }
     updateContextMenu();
 }
@@ -496,6 +496,9 @@ void ComicApplet::updateContextMenu()
 {
     if (mCurrent.id().isEmpty()) {
         mActiveComicModel->clear();
+    }
+
+    if (mCurrent.id().isEmpty() || !mCurrent.ready()) {
         mActionNextNewStripTab->setEnabled(false);
         mActionGoFirst->setEnabled(false);
         mActionGoLast->setEnabled(false);
@@ -799,6 +802,11 @@ bool ComicApplet::isTabHighlighted(const QString &id) const
     return false;
 }
 
+void ComicApplet::loadProviders()
+{
+    mModel->load();
+}
+
 K_PLUGIN_CLASS(ComicApplet)
 
 #include "comic.moc"
