@@ -12,13 +12,16 @@
 #include <QDebug>
 #include <QDesktopServices>
 #include <QGuiApplication>
+#include <QMutex>
 
 #include <cmath>
 
+static QMutex s_initMutex;
+
 K_PLUGIN_CLASS_WITH_JSON(ConverterRunner, "plasma-runner-converter.json")
 
 ConverterRunner::ConverterRunner(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args)
-    : Plasma::AbstractRunner(parent, metaData, args)
+    : AbstractRunner(parent, metaData, args)
 {
     setObjectName(QStringLiteral("Converter"));
 
@@ -26,7 +29,7 @@ ConverterRunner::ConverterRunner(QObject *parent, const KPluginMetaData &metaDat
         "Converts the value of :q: when :q: is made up of "
         "\"value unit [>, to, as, in] unit\". You can use the "
         "Unit converter applet to find all available units.");
-    addSyntax(Plasma::RunnerSyntax(QStringLiteral(":q:"), description));
+    addSyntax(RunnerSyntax(QStringLiteral(":q:"), description));
 }
 
 void ConverterRunner::init()
@@ -39,10 +42,6 @@ void ConverterRunner::init()
     }
     conversionRegex.append(QStringLiteral(" ?> ?"));
     unitSeperatorRegex = QRegularExpression(conversionRegex);
-    valueRegex.optimize();
-    unitSeperatorRegex.optimize();
-
-    insertCompatibleUnits();
 
     actionList = {new QAction(QIcon::fromTheme(QStringLiteral("edit-copy")), i18n("Copy unit and number"), this)};
     setMinLetterCount(2);
@@ -51,12 +50,13 @@ void ConverterRunner::init()
 
 ConverterRunner::~ConverterRunner() = default;
 
-void ConverterRunner::match(Plasma::RunnerContext &context)
+void ConverterRunner::match(RunnerContext &context)
 {
     const QRegularExpressionMatch valueRegexMatch = valueRegex.match(context.query());
     if (!valueRegexMatch.hasMatch()) {
         return;
     }
+
     const QString inputValueString = valueRegexMatch.captured(1);
 
     // Get the different units by splitting up the query with the regex
@@ -64,12 +64,22 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
     if (unitStrings.isEmpty() || unitStrings.at(0).isEmpty()) {
         return;
     }
+
+    // Initialize if not done already
+    {
+        QMutexLocker lock(&s_initMutex);
+        if (!converter) {
+            converter = std::make_unique<KUnitConversion::Converter>();
+            insertCompatibleUnits();
+        }
+    }
+
     // Check if unit is valid, otherwise check for the value in the compatibleUnits map
     QString inputUnitString = unitStrings.first().simplified();
-    KUnitConversion::UnitCategory inputCategory = converter.categoryForUnit(inputUnitString);
+    KUnitConversion::UnitCategory inputCategory = converter->categoryForUnit(inputUnitString);
     if (inputCategory.id() == KUnitConversion::InvalidCategory) {
         inputUnitString = compatibleUnits.value(inputUnitString.toUpper());
-        inputCategory = converter.categoryForUnit(inputUnitString);
+        inputCategory = converter->categoryForUnit(inputUnitString);
         if (inputCategory.id() == KUnitConversion::InvalidCategory) {
             return;
         }
@@ -89,15 +99,15 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
     }
 
     const double numberValue = numberDataPair.second;
-    QList<Plasma::QueryMatch> matches;
+    QList<QueryMatch> matches;
     for (const KUnitConversion::Unit &outputUnit : outputUnits) {
         KUnitConversion::Value outputValue = inputCategory.convert(KUnitConversion::Value(numberValue, inputUnit), outputUnit);
         if (!outputValue.isValid() || inputUnit == outputUnit) {
             continue;
         }
 
-        Plasma::QueryMatch match(this);
-        match.setType(Plasma::QueryMatch::HelperMatch);
+        QueryMatch match(this);
+        match.setType(QueryMatch::HelperMatch);
         match.setIconName(QStringLiteral("accessories-calculator"));
         if (outputUnit.categoryId() == KUnitConversion::CurrencyCategory) {
             match.setText(QStringLiteral("%1 (%2)").arg(outputValue.toString(0, 'f', 2), outputUnit.symbol()));
@@ -113,7 +123,7 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
     context.addMatches(matches);
 }
 
-void ConverterRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
+void ConverterRunner::run(const RunnerContext &context, const QueryMatch &match)
 {
     Q_UNUSED(context)
 
@@ -124,8 +134,11 @@ void ConverterRunner::run(const Plasma::RunnerContext &context, const Plasma::Qu
         QGuiApplication::clipboard()->setText(match.data().toString());
     }
 }
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QPair<bool, double> ConverterRunner::stringToDouble(const QStringRef &value)
+#else
+QPair<bool, double> ConverterRunner::stringToDouble(const QStringView &value)
+#endif
 {
     bool ok;
     double numberValue = locale.toDouble(value, &ok);
@@ -137,7 +150,11 @@ QPair<bool, double> ConverterRunner::stringToDouble(const QStringRef &value)
 
 QPair<bool, double> ConverterRunner::getValidatedNumberValue(const QString &value)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     const auto fractionParts = value.splitRef(QLatin1Char('/'), Qt::SkipEmptyParts);
+#else
+    const auto fractionParts = QStringView(value).split(QLatin1Char('/'), Qt::SkipEmptyParts);
+#endif
     if (fractionParts.isEmpty() || fractionParts.count() > 2) {
         return {false, 0};
     }
@@ -197,14 +214,14 @@ QList<KUnitConversion::Unit> ConverterRunner::createResultUnits(QString &outputU
 
     return units;
 }
+
 void ConverterRunner::insertCompatibleUnits()
 {
     // Add all currency symbols to the map, if their ISO code is supported by backend
     const QList<QLocale> allLocales = QLocale::matchingLocales(QLocale::AnyLanguage, QLocale::AnyScript, QLocale::AnyCountry);
-    KUnitConversion::UnitCategory currencyCategory = converter.category(QStringLiteral("Currency"));
+    KUnitConversion::UnitCategory currencyCategory = converter->category(QStringLiteral("Currency"));
     const QStringList availableISOCodes = currencyCategory.allUnits();
-    QRegularExpression hasCurrencyRegex = QRegularExpression(QStringLiteral("\\p{Sc}"));
-    hasCurrencyRegex.optimize();
+    const QRegularExpression hasCurrencyRegex = QRegularExpression(QStringLiteral("\\p{Sc}")); // clazy:exclude=use-static-qregularexpression
     for (const auto &currencyLocale : allLocales) {
         const QString symbol = currencyLocale.currencySymbol(QLocale::CurrencySymbol);
         const QString isoCode = currencyLocale.currencySymbol(QLocale::CurrencyIsoCode);
@@ -218,7 +235,7 @@ void ConverterRunner::insertCompatibleUnits()
     }
 
     // Add all units as uppercase in the map
-    const auto categories = converter.categories();
+    const auto categories = converter->categories();
     for (const auto &category : categories) {
         const auto allUnits = category.allUnits();
         for (const auto &unit : allUnits) {
