@@ -14,6 +14,7 @@
 #include <QStandardPaths>
 
 #include <KDirWatch>
+#include <KFuzzyMatcher>
 #include <KLocalizedString>
 #include <KNotificationJobUiDelegate>
 
@@ -22,22 +23,16 @@
 K_PLUGIN_CLASS_WITH_JSON(KateSessions, "plasma-runner-katesessions.json")
 
 KateSessions::KateSessions(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args)
-    : Plasma::AbstractRunner(parent, metaData, args)
+    : AbstractRunner(parent, metaData, args)
 {
     setObjectName(QStringLiteral("Kate Sessions"));
 
-    addSyntax(Plasma::RunnerSyntax(QStringLiteral("kate :q:"), i18n("Finds Kate sessions matching :q:.")));
-    addSyntax(Plasma::RunnerSyntax(QStringLiteral("kate"), i18n("Lists all the Kate editor sessions in your account.")));
+    addSyntax(RunnerSyntax(QStringLiteral("kate :q:"), i18n("Finds Kate sessions matching :q:.")));
+    addSyntax(RunnerSyntax(QStringLiteral("kate"), i18n("Lists all the Kate editor sessions in your account.")));
 
     m_sessionsFolderPath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/kate/sessions");
 
     // Initialize watchers and sessions
-    m_sessionWatch = new KDirWatch(this);
-    m_sessionWatch->addDir(m_sessionsFolderPath);
-    connect(m_sessionWatch, &KDirWatch::dirty, this, &KateSessions::loadSessions);
-    connect(m_sessionWatch, &KDirWatch::created, this, &KateSessions::loadSessions);
-    connect(m_sessionWatch, &KDirWatch::deleted, this, &KateSessions::loadSessions);
-    loadSessions();
     setTriggerWords({m_triggerWord});
 }
 
@@ -45,7 +40,7 @@ KateSessions::~KateSessions()
 {
 }
 
-void KateSessions::loadSessions()
+QStringList KateSessions::loadSessions()
 {
     QStringList sessions;
     const QDir sessionsDir(m_sessionsFolderPath);
@@ -55,11 +50,10 @@ void KateSessions::loadSessions()
         sessions.append(QUrl::fromPercentEncoding(sessionFile.baseName().toLocal8Bit()));
     }
 
-    m_sessions = sessions;
-    suspendMatching(m_sessions.isEmpty());
+    return sessions;
 }
 
-void KateSessions::match(Plasma::RunnerContext &context)
+void KateSessions::match(RunnerContext &context)
 {
     QString term = context.query();
     bool listAll = false;
@@ -73,21 +67,54 @@ void KateSessions::match(Plasma::RunnerContext &context)
         return;
     }
 
-    for (const QString &session : std::as_const(m_sessions)) {
-        if (listAll || session.contains(term, Qt::CaseInsensitive)) {
-            Plasma::QueryMatch match(this);
-            match.setType(Plasma::QueryMatch::ExactMatch);
+    // we got here, load sessions now
+    const auto sessions = loadSessions();
+    if (sessions.isEmpty()) {
+        suspendMatching(true);
+        return;
+    }
+
+    QList<QueryMatch> matches;
+    int maxScore = 0;
+
+    for (const QString &session : std::as_const(sessions)) {
+        // Does the query match exactly?
+        // no query = perfect match => list everything
+        if (listAll || session.compare(term, Qt::CaseInsensitive) == 0) {
+            QueryMatch match(this);
+            match.setType(QueryMatch::ExactMatch);
             match.setRelevance(session.compare(term, Qt::CaseInsensitive) == 0 ? 1 : 0.8);
             match.setIconName(m_triggerWord);
             match.setData(session);
             match.setText(session);
             match.setSubtext(i18n("Open Kate Session"));
             context.addMatch(match);
+        } else {
+            // Do fuzzy matching
+            const auto res = KFuzzyMatcher::match(term, session);
+            if (res.matched) {
+                QueryMatch match(this);
+                match.setRelevance(res.score); // store the score here for now
+                match.setIconName(m_triggerWord);
+                match.setData(session);
+                match.setText(session);
+                match.setSubtext(i18n("Open Kate Session"));
+                matches.push_back(match);
+                maxScore = std::max(res.score, maxScore);
+            }
         }
     }
+
+    auto calculate_relevance = [maxScore](double score) {
+        return score / maxScore;
+    };
+    for (auto &match : matches) {
+        match.setRelevance(calculate_relevance(match.relevance()));
+    }
+    context.addMatches(matches);
 }
 
-void KateSessions::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
+void KateSessions::run(const RunnerContext &context, const QueryMatch &match)
 {
     Q_UNUSED(context)
 
