@@ -29,6 +29,10 @@ PotdClient::PotdClient(const KPluginMetaData &metadata, const QVariantList &args
 
 void PotdClient::updateSource(bool refresh)
 {
+    if (m_loading) {
+        return;
+    }
+
     setLoading(true);
 
     // Check whether it is cached already...
@@ -52,6 +56,7 @@ void PotdClient::updateSource(bool refresh)
     const auto pluginResult = KPluginFactory::instantiatePlugin<PotdProvider>(m_metadata, this, m_args);
 
     if (pluginResult) {
+        qCDebug(WALLPAPERPOTD) << "Downloading wallpaper from" << m_identifier << m_args;
         connect(pluginResult.plugin, &PotdProvider::finished, this, &PotdClient::slotFinished);
         connect(pluginResult.plugin, &PotdProvider::error, this, &PotdClient::slotError);
     } else {
@@ -172,6 +177,7 @@ PotdEngine::PotdEngine(QObject *parent)
     int interval = QDateTime::currentDateTime().msecsTo(QDate::currentDate().addDays(1).startOfDay()) + 1000;
     m_checkDatesTimer.setInterval(std::max(interval, 60 * 1000));
     m_checkDatesTimer.start();
+    qCDebug(WALLPAPERPOTD) << "Time to next update (h):" << m_checkDatesTimer.interval() / 1000.0 / 60.0 / 60.0;
 
     // Sleep checker
     QDBusConnection::systemBus().connect(QStringLiteral("org.freedesktop.login1"),
@@ -241,6 +247,10 @@ void PotdEngine::updateSource(bool refresh)
     m_lastUpdateSuccess = true;
 
     for (const auto &pr : std::as_const(m_clientMap)) {
+        if (pr.second.client->m_loading) {
+            continue;
+        }
+
         connect(pr.second.client, &PotdClient::done, this, &PotdEngine::slotDone);
         m_updateCount++;
         qCDebug(WALLPAPERPOTD) << pr.second.client->m_metadata.value(QStringLiteral("X-KDE-PlasmaPoTDProvider-Identifier")) << "starts updating wallpaper.";
@@ -272,7 +282,7 @@ void PotdEngine::slotDone(PotdClient *client, bool success)
             m_checkDatesTimer.setInterval(10min);
         }
         m_checkDatesTimer.start();
-        qCDebug(WALLPAPERPOTD) << "Time to next update (ms):" << m_checkDatesTimer.interval();
+        qCDebug(WALLPAPERPOTD) << "Time to next update (h):" << m_checkDatesTimer.interval() / 1000.0 / 60.0 / 60.0;
     }
 }
 
